<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Handler_AdmAccounts extends Controller_Handler
{
    public function before() {
        parent::before();
        
        $this->user->checkAccess(User::RIGHTS_ADMIN | User::RIGHTS_DEAN);
    }

    private function defineFaculty($recievedFacultyID) {
        $faculty = User::instance()->Faculty;
        if(!empty($recievedFacultyID))
            $faculty = Model_Faculty::with($recievedFacultyID);
        return $faculty;
    }

    public function action_getActivationForTeachers() {
        $faculty = $this->defineFaculty($this->post['facultyID']);
        $departments = $faculty->getDepartments();
        $departmentsHandled = [];
        $i = 0;

        foreach ($departments as $department) {
            $accounts = Model_Teachers::ofDepartment($department['ID']);
            if ($accounts == null)
                continue;

            $i++;
            $j = 0;
            $departmentsHandled[$i]['Name'] = $department['Name'];

            foreach ($accounts as $row) {
                # TODO: db query in loop
                $accInfo = Model_Account::with($row['AccountID']);
                if ($code = $accInfo->get('Code')) {
                    $j++;
                    $departmentsHandled[$i]['People'][$j]['LastName'] = $row['LastName'];
                    $departmentsHandled[$i]['People'][$j]['FirstName'] = $row['FirstName'];
                    $departmentsHandled[$i]['People'][$j]['SecondName'] = $row['SecondName'];
                    $departmentsHandled[$i]['People'][$j]['ActivationCode'] = $code;
                }
            }

            if ($j == 0) {
                $departmentsHandled[$i] = [];
                $i--;
            }
        }

        if ($i > 0) {
            $file = PDFCreator::CreatePDF(
                'pdf/activationTeacher',
                'static/css/pdf/activationTemplate.css', [
                    'Departments' => $departmentsHandled,
                    'System'      => [
                        'Title'   => ASSEMBLY_SYSTEM_NAME,
                        'Version' => ASSEMBLY_VERSION
                    ]
                ]);

            $headers = PDFCreator::GetHeaders($file, 'ActivationTeachers' . $faculty->ID);
            foreach ($headers as $key => $value) {
                $this->response->headers($key, $value);
            }

            $this->response->body($file);
        } else {
            throw HTTP_Exception::factory(400, 'No data found');
        }
    }

    public function action_getActivationForStudents() {
        $faculty = $this->defineFaculty($this->post['facultyID']);
        $groups = $faculty->getGroups($this->post['gradeID']);

        $groupsHandled = [];
        $i = 0;

        $semester = User::instance()->SemesterID;
        foreach ($groups as $group) {
            $accounts = Model_Students::ofGroup($group['ID'], $semester);
            if (empty($accounts))
                continue;

            $i++;
            $j = 0;

            $groupsHandled[$i]['Grade'] = $_POST['gradeID'];
            $groupsHandled[$i]['Num'] = $group['GroupNum'];
            $groupsHandled[$i]['Spec'] = $group['SpecName'];

            foreach ($accounts as $row) {
                # TODO: db query in loop
                $accInfo = Model_Account::with($row['AccountID']);
                if ($code = $accInfo['Code']) {
//                  if ($code = $accInfo->getActivationCode()) {
                        $j++;
                    $groupsHandled[$i]['People'][$j]['LastName'] = $row['LastName'];
                    $groupsHandled[$i]['People'][$j]['FirstName'] = $row['FirstName'];
                    $groupsHandled[$i]['People'][$j]['SecondName'] = $row['SecondName'];
                    $groupsHandled[$i]['People'][$j]['ActivationCode'] = $code;
                }
            }

            if ($j == 0) {
                $groupsHandled[$i] = [];
                $i--;
            }
        }

        if ($i > 0) {
            $file = PDFCreator::CreatePDF(
                'pdf/activationStudent',
                'static/css/pdf/activationTemplate.css', [
                    'Groups' => $groupsHandled,
                    'System' => [
                        'Title'   => ASSEMBLY_SYSTEM_NAME,
                        'Version' => ASSEMBLY_VERSION
                    ]
                ]);

            $headers = PDFCreator::GetHeaders($file, 'ActivationStudents');
            foreach ($headers as $key => $value) {
                $this->response->headers($key, $value);
            }

            $this->response->body($file);
        } else {
            throw HTTP_Exception::factory(400, 'No data found');
        }
    }
}
